﻿using System;
using System.Collections.Generic;
using AddressSolution.Common;

namespace AddressSolution.AddressRules
{
    public static class RuleMunicipality
    {
        public static string GetMunicipality(string address)
        {
            List<string> streetTypes = ValidCollections.GetStreetTypes();

            // replace comma with space
            string addressLine = address.Trim().Replace(",", " ");

            int streetTypeFound = 0;
            int streetTypeLength = 0;
            foreach (var stType in streetTypes)
            {
                streetTypeFound = addressLine.IndexOf(stType, StringComparison.OrdinalIgnoreCase);
                if (streetTypeFound > -1)
                {
                    streetTypeLength = stType.Length;
                    break;
                }
            }

            // Municipality is before the province
            List<string> provinces = ValidCollections.GetProvinces();

            int startOfProvince = 0;
            foreach (var province in provinces)
            {
                startOfProvince = addressLine.IndexOf(province, StringComparison.OrdinalIgnoreCase);
                if (startOfProvince > -1) break;
            }

            string municipality = string.Empty;
            try
            {
                municipality = addressLine.Substring(streetTypeFound + streetTypeLength - 1, startOfProvince - (streetTypeFound + streetTypeLength - 1));
            }
            catch (Exception)
            {
                // can't parse Municipality
                Console.WriteLine(address);
            }

            // Remove any StreetDirection if present
            List<string> streetDirections = ValidCollections.GetStreetDirections();
            string municipalityFinal = municipality;
            foreach (var streetDirection in streetDirections)
            {
                int foundIndex = municipality.IndexOf(streetDirection, StringComparison.OrdinalIgnoreCase);
                if (foundIndex > -1)
                {
                    municipalityFinal = municipality.Remove(foundIndex, streetDirection.Length);
                    break;
                }
            }

            return municipalityFinal.Trim();
        }
    }
}