﻿using System.Collections.Generic;
using System.Linq;
using System.Runtime.Caching;
using AddressSolution.Dao;

namespace AddressSolution.Common
{
    public static class ValidCollections
    {
        // Create a MemoryCache instance so we can avoid unnecessary DB calls
        private static ObjectCache objectCache = MemoryCache.Default;

        // Declare once
        private static AddressDBContext addressDBContext = new AddressDBContext();

        public static List<string> GetStreetTypes()
        {
            //List<string> streetTypes = new List<string>()
            //{
            //    " RUE "," AVE "," BLVD "," PL "," DR "," CRES ", " ST ", " RD ", " ROAD ", " CH ", " STREET ", " CRT ",
            //    " WAY ", " AV ", " BOUL ", " LANE ", " CIR ", " SQ ", " CLSE ", " RD. "
            //};

            if (objectCache.Contains("streetTypes"))
            {
                List<string> streetTypes = objectCache.Get("streetTypes") as List<string>;
                return streetTypes;
            }
            else
            {
                List<string> streetTypes = addressDBContext.StreetTypes.Select(p => p.StreetType).Select(p => p.Replace(p, " " + p + " ")).ToList();
                objectCache.Add("streetTypes", streetTypes, new CacheItemPolicy());
                return streetTypes;
            }
        }

        public static List<string> GetStreetDirections()
        {
            //List<string> streetDirections = new List<string>()
            //{
            //    " E "," W "," N "," S "," NE "," NW ", " SE ", " SW "
            //};

            if (objectCache.Contains("streetDirections"))
            {
                List<string> streetDirections = objectCache.Get("streetDirections") as List<string>;
                return streetDirections;
            }
            else
            {
                List<string> streetDirections = addressDBContext.StreetDirections.Select(p => p.StreetDirection).Select(p => p.Replace(p, " " + p + " ")).ToList();
                objectCache.Add("streetDirections", streetDirections, new CacheItemPolicy());
                return streetDirections;
            }
        }

        public static List<string> GetSuites()
        {
            //List<string> suites = new List<string>()
            //{
            //    " APT "," UNIT "," SUITE "
            //};

            if (objectCache.Contains("suites"))
            {
                List<string> suites = objectCache.Get("suites") as List<string>;
                return suites;
            }
            else
            {
                List<string> suites = addressDBContext.Suites.Select(p => p.Suite).Select(p => p.Replace(p, " " + p + " ")).ToList();
                objectCache.Add("suites", suites, new CacheItemPolicy());
                return suites;
            }
        }

        public static List<string> GetProvinces()
        {
            //List<string> provinces = new List<string>()
            //{
            //    " AB ", " BC ", " MB ", " NB ", " NL ", " NT ", " NS ", " NU ", " ON ", " PE ", " QC ", " SK ", " YT ",
            //    " ONT ", " ONTARIO ", " QUEBEC ", " QUE "
            //};

            if (objectCache.Contains("provinces"))
            {
                List<string> provinces = objectCache.Get("provinces") as List<string>;
                return provinces;
            }
            else
            {
                List<string> provinces = addressDBContext.Provinces.Select(p => p.Province).Select(p => p.Replace(p, " " + p + " ")).ToList();
                objectCache.Add("provinces", provinces, new CacheItemPolicy());
                return provinces;
            }
        }
    }
}
