﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using ArchieSolution.Dao;
using ArchieSolution.Extensions;
using ArchieSolution.Models;

namespace ArchieSolution.Controllers
{
    public class HomeController : ApiController
    {
        // example: http://localhost:64695/api/Uom
        [HttpGet]
        [Route("api/Uom")]
        public HttpResponseMessage GetUom()
        {
            HttpResponseMessage httpResponseMessage = Request.CreateResponse(HttpStatusCode.OK, Enum.GetNames(typeof(Uom)));
            return httpResponseMessage;
        }

        // example: http://localhost:64695/api/GetSampleArchies
        [HttpGet]
        [Route("api/GetSampleArchies")]
        public HttpResponseMessage GetSampleArchies()
        {
            ArchieDao archieDao = new ArchieDao();
            List<Archie> archies = archieDao.GetSampleArchies();

            HttpResponseMessage httpResponseMessage = Request.CreateResponse(HttpStatusCode.OK, archies);
            return httpResponseMessage;
        }

        // example: http://localhost:64695/api/values/archie
        // header: Content-Type: application/json
        // payload: {"Dimension":{"Uom":"Inches","Height":400.25,"Width":505.25,"Length":600.50,"Girth":100.0,"WingHandValue":200.0},"Skeleton":{"Wingbone":2,"HandThing":2,"Skull":1,"Teeth":200,"Feet":2,"Tail":1,"Spine":1},"SpecimenName":"Big Bird","Location":{"Latitude":52.5243700,"Longitude":13.4105300},"TotalWingSpan":0}
        [HttpPost]
        [Route("api/values/archie")]
        public HttpResponseMessage PostArchie(Archie postArchie)
        {
            var dimensionErrorList = postArchie.Dimension.Validate();
            var skeletonErrorList = postArchie.Skeleton.Validate();

            if (dimensionErrorList.Count > 0 || skeletonErrorList.Count > 0 || postArchie.SpecimenName == null)
            { 
                dimensionErrorList.AddRange(skeletonErrorList);
                throw new Exception("Input Error:\n" + string.Join("\n", dimensionErrorList.ToArray()));
            }

            ArchieDao archieDao = new ArchieDao();
            archieDao.Save(postArchie);

            HttpResponseMessage httpResponseMessage = Request.CreateResponse(HttpStatusCode.OK, postArchie);
            return httpResponseMessage;
        }

        // example: http://localhost:64695/api/GetAllArchies
        [HttpGet]
        [Route("api/GetAllArchies")]
        public HttpResponseMessage GetAllArchies()
        {
            ArchieDao archieDao = new ArchieDao();
            List<Archie> archies = archieDao.GetAllArchies();

            HttpResponseMessage httpResponseMessage = Request.CreateResponse(HttpStatusCode.OK, archies);
            return httpResponseMessage;
        }
    }
}
