﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Runtime.Caching;
using ArchieSolution.Models;

namespace ArchieSolution.Dao
{
    public class ArchieDao
    {
        // To see the JSON payload structure
        public List<Archie> GetSampleArchies()
        {
            List<Archie> archies = new List<Archie>();


            Dimension dimension = new Dimension();
            dimension.Uom = Uom.Inches;
            dimension.Height = 400.25m;
            dimension.Width = 505.25m;
            dimension.Length = 600.50m;
            dimension.Girth = 100m;
            dimension.WingHandValue = 200m;
            var dimensionErrorList = dimension.Validate();

            Skeleton skeleton = new Skeleton();
            skeleton.Wingbone = 2;
            skeleton.HandThing = 2;
            skeleton.Skull = 1;
            skeleton.Teeth = 200;
            skeleton.Feet = 2;
            skeleton.Tail = 1;
            skeleton.Spine = 1;
            skeleton.Validate();
            var skeletonErrorList = skeleton.Validate();
            if (dimensionErrorList.Count > 0 || skeletonErrorList.Count > 0)
            {
                dimensionErrorList.AddRange(skeletonErrorList);
                throw new Exception("Input Error:\n" + string.Join("\n", dimensionErrorList.ToArray()));
            }

            Location location = new Location();
            location.Latitude = 52.5243700m;
            location.Longitude = 13.4105300m;

            Archie archie1 = new Archie(dimension, skeleton, location, "Big Bird 1");
            archies.Add(archie1);

            Archie archie2 = new Archie(dimension, skeleton, location, "Big Bird 2");
            archies.Add(archie2);


            return archies;
        }

        // Create a MemoryCache instance so we can simulate
        ObjectCache cacheService = MemoryCache.Default;
        public void Save(Archie archie)
        {
            // Normally we would save this to a DB
            cacheService.Add(archie.SpecimenName, archie, new CacheItemPolicy());
        }

        public List<Archie> GetAllArchies()
        {
            // Normally we would retrieve this from a DB
            List<Archie> archies = new List<Archie>();
            foreach (var item in MemoryCache.Default)
            {
                Archie archie = item.Value as Archie;
                archies.Add(archie);
            }
            return archies;
        }
    }
}